searchNodes=[{"doc":"nosedrum is a command framework for use with the excellent nostrum library. It contains behaviour specifications for easily implementing command handling for both Discord's application commands and the traditional text-based commands in your bot along with other conveniences to ease creating an interactive bot. nosedrum s provided implementations are largely based off what was originally written for bolt . bolt also contains around 57 commands based off the Nosedrum.Command behaviour that you can explore if you're looking for inspiration. The application command related parts of the framework consist of two parts: Nosedrum.ApplicationCommand , the behaviour that all application commands must implement. Nosedrum.Invoker , the behaviour for any slash command invoker. A default implementation provided by nosedrum resides at Nosedrum.Invoker.Dispatcher . The traditional command processing related parts of the framework consists of three parts: Nosedrum.Command , the behaviour that all commands must implement. Nosedrum.Invoker , the behaviour of command processors. Command processors take a message, look it up in the provided storage implementation, and invoke commands as required. nosedrum ships with an implementation of this based on bolt's original command parser named Nosedrum.Invoker.Split . Nosedrum.Storage , the behaviour of command storages. Command storages allow for fast and simple lookups of commands and command groups and store command names along with their corresponding Nosedrum.Command implementations internally. An ETS-based command storage implementation is provided with Nosedrum.Storage.ETS . Additionally, the following utilities are provided: Nosedrum.Converters , functions for converting parts of messages to objects from Nostrum such as channels, members, and roles. Nosedrum.MessageCache , a behaviour for defining message caches, along with an ETS-based and an Agent-based implementation. Simply add :nosedrum to your mix.exs : def deps do [ { :nosedrum , &quot;~&gt; 0.4&quot; } , # To use the GitHub version of Nostrum: # {:nostrum, github: &quot;Kraigie/nostrum&quot;, override: true} ] end Getting started For getting started with application commands, see the Nosedrum.ApplicationCommand module. To write a traditional command, your commands need to implement the Nosedrum.Command behaviour. As a simple example, let's reimplement ed . defmodule MyBot.Cogs.Ed do @behaviour Nosedrum.Command alias Nostrum.Api @impl true def usage , do : [ &quot;ed [-GVhs] [-p string] [file]&quot; ] @impl true def description , do : &quot;Ed is the standard text editor.&quot; @impl true def predicates , do : [ ] @impl true def command ( msg , _args ) do { :ok , _msg } = Api . create_message ( msg . channel_id , &quot;?&quot; ) end end With your commands defined, choose a Nosedrum.Storage implementation and add it to your application callback. We will use the included Nosedrum.Storage.ETS implementation here, but feel free to write your own: defmodule MyBot.Application do use Application def start ( _type , _args ) do children = [ Nosedrum.Storage.ETS , MyBot.Consumer ] options = [ strategy : :one_for_one , name : MyBot.Supervisor ] Supervisor . start_link ( children , options ) end end Finally, we hook things up in our consumer: we will load commands once the bot is ready, and invoke the command invoker on each message. defmodule MyBot.Consumer do alias Nosedrum.Invoker.Split , as : CommandInvoker alias Nosedrum.Storage.ETS , as : CommandStorage use Nostrum.Consumer @commands %{ &quot;ed&quot; =&gt; MyBot.Cogs.Ed } def handle_event ( { :READY , _data , _ws_state } ) do Enum . each ( @commands , fn { name , cog } -&gt; CommandStorage . add_command ( [ name ] , cog ) end ) end def handle_event ( { :MESSAGE_CREATE , msg , _ws_state } ) do CommandInvoker . handle_message ( msg , CommandStorage ) end def handle_event ( _data ) , do : :ok end That's all we need to get started with. If you want to customize your bot's prefix, set the nosedrum.prefix configuration variable: config :nosedrum , prefix : System . get_env ( &quot;BOT_PREFIX&quot; ) || &quot;.&quot; If no value is configured, the default prefix used depends on the chosen command invoker implementation. Nosedrum.Invoker.Split defaults to . .","ref":"Nosedrum.html","title":"Nosedrum","type":"module"},{"doc":"The application command behaviour specifies the interface that a slash, user, or message command module should implement. Like regular commands, application command modules are stateless on their own. Implementations of the callbacks defined by this behaviour are invoked from other modules/functions, notably a Nosedrum.Interactor . The types defined in this module reflect the official Application Command docs . Example Slash Command This command echos the passed message back to the user. # In your application command module file, e.g. ./lib/my_app/commands/echo.ex defmodule MyApp.Commands.Echo do @behaviour Nosedrum.ApplicationCommand @impl true def description ( ) do &quot;Echos a message.&quot; end @impl true def command ( interaction ) do [ %{ name : &quot;message&quot; , value : message } ] = interaction . data . options [ content : message , ephemeral? : true ] end @impl true def type ( ) do :slash end @impl true def options ( ) do [ %{ type : :string , name : &quot;message&quot; , description : &quot;The message for the bot to echo.&quot; , required : true } ] end end # In your Nostrum.Consumer file, e.g. ./lib/my_app/consumer.ex defmodule MyApp.Consumer do use Nostrum.Consumer # ... # You may use `:global` instead of a guild id at GUILD_ID_HERE, but note # that global commands could take up to an hour to become available. def handle_event ( { :READY , _data , _ws_state } ) do case Nosedrum.Interactor.Dispatcher . add_command ( &quot;echo&quot; , MyApp.Commands.Echo , GUILD_ID_HERE ) do { :ok , _ } -&gt; IO . puts ( &quot;Registered Echo command.&quot; ) e -&gt; IO . inspect ( e , label : &quot;An error occurred registering the Echo command&quot; ) end end end You will also need to start the Nosedrum.Interactor.Dispatcher as part of your supervision tree, for example, by adding this to your application start function: # ./lib/my_app/application.ex defmodule MyApp.Application do # ... def start ( type , args ) do children = [ # ... { Nosedrum.Interactor.Dispatcher , name : Nosedrum.Interactor.Dispatcher } , ] options = [ strategy : :rest_for_one , name : Bolt.Supervisor ] Supervisor . start_link ( children , options ) end end","ref":"Nosedrum.ApplicationCommand.html","title":"Nosedrum.ApplicationCommand","type":"behaviour"},{"doc":"Execute the command invoked by the given Nostrum.Struct.Interaction.t/0 . Returns a response/0 Example defmodule MyApp.MyCommand do @behaviour Nosedrum.ApplicationCommand # ... @impl true def command ( interaction ) do %{ name : opt_name } = List . first ( interaction . data . options ) [ content : &quot;Hello World \#{ opt_name } !&quot; ] end end","ref":"Nosedrum.ApplicationCommand.html#c:command/1","title":"Nosedrum.ApplicationCommand.command/1","type":"callback"},{"doc":"Returns a description of the command. Used when registering the command with Discord. This is what the user will see in the autofill command-selection menu. Example def description , do : &quot;This is a command description.&quot;","ref":"Nosedrum.ApplicationCommand.html#c:description/0","title":"Nosedrum.ApplicationCommand.description/0","type":"callback"},{"doc":"An optional callback that returns a list of options (arguments) that the command takes. Used when registering the command with Discord. Only valid for CHAT_INPUT application commands, aka slash commands. Read more in the official Application Command documentation . Example options callback for a &quot;/role&quot; command def options , do : [ %{ type : :user , name : &quot;user&quot; , description : &quot;The user to assign the role to.&quot; , required : true # Defaults to false if not specified. } , %{ type : :role , name : &quot;role&quot; , description : &quot;The role to be assigned.&quot; , required : true , choices : [ %{ name : &quot;Event Notifications&quot; , value : 123456789123456789 # A role ID, passed to your `command/1` callback via the Interaction struct. } , %{ name : &quot;Announcements&quot; , value : 123456789123456790 } ] } ]","ref":"Nosedrum.ApplicationCommand.html#c:options/0","title":"Nosedrum.ApplicationCommand.options/0","type":"callback"},{"doc":"Returns one of :slash , :message , or :user , indicating what kind of application command this module represents.","ref":"Nosedrum.ApplicationCommand.html#c:type/0","title":"Nosedrum.ApplicationCommand.type/0","type":"callback"},{"doc":"A choice for an option. See callback options/0 documentation for examples.","ref":"Nosedrum.ApplicationCommand.html#t:choice/0","title":"Nosedrum.ApplicationCommand.choice/0","type":"type"},{"doc":"An option (argument) for an application command. See callback options/0 documentation for examples.","ref":"Nosedrum.ApplicationCommand.html#t:option/0","title":"Nosedrum.ApplicationCommand.option/0","type":"type"},{"doc":"A keyword list of fields to include in the interaction response, after running the command/1 callback. If :type is not specified, it will default to :channel_message_with_source , though one of either :embeds or :content must be present. Example def command ( interaction ) do # Since `:content` is included, Nosedrum will infer `type: :channel_message_with_source` [ content : &quot;Hello, world!&quot; , ephemeral? : true , allowed_mentions : [ &quot;users&quot; , &quot;roles&quot; ] ] end","ref":"Nosedrum.ApplicationCommand.html#t:response/0","title":"Nosedrum.ApplicationCommand.response/0","type":"type"},{"doc":"A field in a keyword list interaction response. Special notes: :type is required, unless :content or :embeds is present, in which case it defaults to :channel_message_with_source . :allowed_mentions is a list that should contain &quot;users&quot;, &quot;roles&quot;, and/or &quot;everyone&quot;, or be empty.","ref":"Nosedrum.ApplicationCommand.html#t:response_field/0","title":"Nosedrum.ApplicationCommand.response_field/0","type":"type"},{"doc":"","ref":"Nosedrum.ApplicationCommand.html#t:response_type/0","title":"Nosedrum.ApplicationCommand.response_type/0","type":"type"},{"doc":"Interactors take the role of both Nosedrum.Invoker and Nosedrum.Storage when it comes to Discord's Application Commands. An Interactor handles incoming Nostrum.Struct.Interaction.t/0 s, invoking Nosedrum.ApplicationCommand.command/1 callbacks and responding to the Interaction. In addition to tracking commands locally for the bot, an Interactor is responsible for registering an Application Command with Discord when add_command/4 or remove_command/4 is called.","ref":"Nosedrum.Interactor.html","title":"Nosedrum.Interactor","type":"behaviour"},{"doc":"Add a new command under the given name or application command path. Returns :ok if successful, and {:error, reason} otherwise. If the command already exists, it will be overwritten.","ref":"Nosedrum.Interactor.html#c:add_command/4","title":"Nosedrum.Interactor.add_command/4","type":"callback"},{"doc":"Handle an Application Command invocation. This callback should be invoked upon receiving an interaction via the :INTERACTION_CREATE event. Example using Nosedrum.Interactor.Dispatcher : # In your `Nostrum.Consumer` file: def handle_event ( { :INTERACTION_CREATE , interaction , _ws_state } ) do IO . puts &quot;Got interaction&quot; Nosedrum.Interactor.Dispatcher . handle_interaction ( interaction ) end","ref":"Nosedrum.Interactor.html#c:handle_interaction/2","title":"Nosedrum.Interactor.handle_interaction/2","type":"callback"},{"doc":"Remove the command under the given name or application command path. Returns :ok if successful, and {:error, reason} otherwise. If the command does not exist, no error should be returned.","ref":"Nosedrum.Interactor.html#c:remove_command/4","title":"Nosedrum.Interactor.remove_command/4","type":"callback"},{"doc":"Responds to an Interaction with the values in the given Nosedrum.ApplicationCommand.response/0 . Returns {:ok} if successful, and a Nostrum.Api.error/0 otherwise.","ref":"Nosedrum.Interactor.html#respond/2","title":"Nosedrum.Interactor.respond/2","type":"function"},{"doc":"Defines a structure of commands, subcommands, subcommand groups, as outlined in the official documentation . Note that Discord only supports nesting 3 levels deep, like command -&gt; subcommand group -&gt; subcommand . Example path: %{ { &quot;castle&quot; , MyApp.CastleCommand . description ( ) } =&gt; %{ { &quot;prepare&quot; , &quot;Prepare the castle for an attack.&quot; } =&gt; [ ] , { &quot;open&quot; , &quot;Open up the castle for traders and visitors.&quot; } =&gt; [ ] , # ... } }","ref":"Nosedrum.Interactor.html#t:application_command_path/0","title":"Nosedrum.Interactor.application_command_path/0","type":"type"},{"doc":"","ref":"Nosedrum.Interactor.html#t:command_scope/0","title":"Nosedrum.Interactor.command_scope/0","type":"type"},{"doc":"The name or pid of the Interactor process.","ref":"Nosedrum.Interactor.html#t:name_or_pid/0","title":"Nosedrum.Interactor.name_or_pid/0","type":"type"},{"doc":"An implementation of Nosedrum.Interactor , dispatching Application Command Interactions to the appropriate modules.","ref":"Nosedrum.Interactor.Dispatcher.html","title":"Nosedrum.Interactor.Dispatcher","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Nosedrum.Interactor.Dispatcher.html#child_spec/1","title":"Nosedrum.Interactor.Dispatcher.child_spec/1","type":"function"},{"doc":"","ref":"Nosedrum.Interactor.Dispatcher.html#start_link/1","title":"Nosedrum.Interactor.Dispatcher.start_link/1","type":"function"},{"doc":"Conversion from command arguments to various types. This module provides an interface to the individual converter modules. Most converter functions related to Discord itself take a guild_id which is used for loading the guild from the cache. If the guild could not be load from the cache, implementations will usually attempt to fetch the relevant data from the API.","ref":"Nosedrum.Converters.html","title":"Nosedrum.Converters","type":"module"},{"doc":"Convert the given text to a Nostrum.Struct.Channel.t/0 . Lookup is attempted in the following order: by direct ID, such as 9999 by mention, such as &lt;#9999&gt; by name, such as mod-log","ref":"Nosedrum.Converters.html#to_channel/2","title":"Nosedrum.Converters.to_channel/2","type":"function"},{"doc":"Convert the given text to a Nostrum.Struct.Guild.Member.t/0 . Lookup is attempted in the following order: by direct ID, such as 1231321 by mention, such as &lt;@1231321&gt; , &lt;@!1231321&gt; by name#discrim combination, such as Jimmy#9999 by name, such as Jimmy by nickname, such as SuperJimmy Note that name and nickname lookups may not be 100% accurate: if there are multiple users with the same name on the server, the first one found will be used.","ref":"Nosedrum.Converters.html#to_member/2","title":"Nosedrum.Converters.to_member/2","type":"function"},{"doc":"Convert the given text to a Nostrum.Struct.Guild.Role.t/0 . Lookup is attempted in the following order: by direct ID, such as 5555 by mention, such as &lt;@&amp;5555&gt; by name, such as Bots The optional ilike argument determines whether the role name search should be case-insensitive. This is useful if your users are lazy and you want to save them from holding down an extra button on their keyboard.","ref":"Nosedrum.Converters.html#to_role/3","title":"Nosedrum.Converters.to_role/3","type":"function"},{"doc":"User interaction helpers that don't fit into their own module.","ref":"Nosedrum.Helpers.html","title":"Nosedrum.Helpers","type":"module"},{"doc":"Escape @everyone and @here mentions in the given content with a zero-width space character. Example iex&gt; Nosedrum.Helpers . escape_server_mentions ( &quot;hello world&quot; ) &quot;hello world&quot; iex&gt; Nosedrum.Helpers . escape_server_mentions ( &quot;hello @everyone @here&quot; ) &quot;hello @​everyone @​here&quot; # No space to be seen, but no mention either!","ref":"Nosedrum.Helpers.html#escape_server_mentions/1","title":"Nosedrum.Helpers.escape_server_mentions/1","type":"function"},{"doc":"Try to split the given text using OptionParser.split/1 and fall back to String.split/1 if not applicable. OptionParser.split/1 raises a RuntimeError if the text it is given contains opening quotation marks that were not closed again. In other cases, it is desirable to split a string into quoted parts, for example: iex&gt; Nosedrum.Helpers . quoted_split ( &quot;echo \\&quot; hello bot \\&quot; &quot; ) [ &quot;echo&quot; , &quot;hello bot&quot; ] iex&gt; Nosedrum.Helpers . quoted_split ( &quot;echo \\&quot; hello bot&quot; ) [ &quot;echo&quot; , &quot; \\&quot; hello&quot; , &quot;bot&quot; ]","ref":"Nosedrum.Helpers.html#quoted_split/1","title":"Nosedrum.Helpers.quoted_split/1","type":"function"},{"doc":"Built-in command predicates and predicate evaluation. Nosedrum.Command.predicates/0 allows commands to specify a set of conditions that must be met before a command is invoked, effectively allowing you to deny access to certain access. For instance, allowing every user on your server to issue a ban command may not be desirable. For using the Nosedrum.Command.predicates/0 callback, you can either define your own predicates or use those provided by this module .","ref":"Nosedrum.Predicates.html","title":"Nosedrum.Predicates","type":"module"},{"doc":"Lazily evaluate all given predicates and return the result. While usually used by command invokers, you can use this function if you need to manually evaluate the result of predicates. Return value If all predicates returned :passthrough , :passthrough is returned. Otherwise, the error of the first predicate returning one is returned.","ref":"Nosedrum.Predicates.html#evaluate/2","title":"Nosedrum.Predicates.evaluate/2","type":"function"},{"doc":"Check whether the command was invoked on a guild. Note that has_permission/1 already checks whether a command was invoked on a guild and this predicate does not need to be stack with it.","ref":"Nosedrum.Predicates.html#guild_only/1","title":"Nosedrum.Predicates.guild_only/1","type":"function"},{"doc":"Check whether the message author has the given permission . This does not directly return an evaluation_result/0 : it returns a function allowing you to use this in your Nosedrum.Command.predicates/0 callback. When evaluation fails, an error with a description of the required permission is returned, or another error when the permission could not be checked on the author - for example, because the guild or member was not found in the cache. Example defmodule MyBot.Cogs.Ban do @behaviour Nosedrum.Command def usage , do : [ &quot;ban &lt;member&gt;&quot; ] def description , do : &quot;Ban the given `member`.&quot; def predicates , do : [ Nosedrum.Predicates . has_permission ( :ban_members ) ] def command ( msg , [ target ] ) do # ... 🔨 end end","ref":"Nosedrum.Predicates.html#has_permission/1","title":"Nosedrum.Predicates.has_permission/1","type":"function"},{"doc":"The result of a predicate evaluation, determining whether the command invocation should proceed or abort. The values represent the following: :passthrough if the predicate permits the command to be invoked. {:noperm, any()} if the predicate does not permit the command to be invoked. {:error, any()} if the predicate could not determine its result. On :passthrough , invokers will continue with predicate checking and finally invoke the command. On :noperm or :error , command execution is aborted.","ref":"Nosedrum.Predicates.html#t:evaluation_result/0","title":"Nosedrum.Predicates.evaluation_result/0","type":"type"},{"doc":"A condition that must pass before a command is invoked. Example defmodule MyBot.Predicates do alias Nostrum.Struct.Message def guild_only ( % Message { guild_id : nil } ) do { :error , &quot;This command can only be used on guilds.&quot; } end def guild_only ( _msg ) , do : :passthrough end","ref":"Nosedrum.Predicates.html#t:predicate/0","title":"Nosedrum.Predicates.predicate/0","type":"type"},{"doc":"The command behaviour specifies the interface that a command module should implement. Command modules are stateless by themselves, if you want to capture state, drop it in a different delegated module. Commands by themselves do not implement any logic for determining whether they were invoked. This is done by a command processor module, which also maps command strings to the command modules to be invoked. Example defmodule MyBot.Cogs.Echo do @behaviour Nosedrum.Command @moduledoc false @impl true def usage , do : [ &quot;echo &lt;text...&gt;&quot; ] @impl true def description , do : &quot;display a line of text&quot; @impl true def parse_args ( args ) , do : Enum . join ( args , &quot; &quot; ) @impl true def predicates , do : [ ] @impl true def command ( msg , &quot;&quot; ) do response = &quot;Need something to echo&quot; { :ok , _msg } = Nostrum.Api . create_message ( msg . channel_id , response ) end def command ( msg , text ) do { :ok , _msg } = Nostrum.Api . create_message ( msg . channel_id , text ) end end","ref":"Nosedrum.Command.html","title":"Nosedrum.Command","type":"behaviour"},{"doc":"An optional callback that returns a list of aliases for the command. Nosedrum.Storage implementations use this function when adding/removing a command with add_command/2 or remove_command/2 . If any of the aliases returned conflict with existing commands/aliases, they will be overwritten.","ref":"Nosedrum.Command.html#c:aliases/0","title":"Nosedrum.Command.aliases/0","type":"callback"},{"doc":"Execute the command invoked by the given Nostrum.Struct.Message.t/0 . The second parameter is a list of arguments with the command (or subcommand) along with the bot prefix removed. If the command defines parse_args/1 , the returned value of that function will be passed instead (marked as any() here). The return value of this function is unused.","ref":"Nosedrum.Command.html#c:command/2","title":"Nosedrum.Command.command/2","type":"callback"},{"doc":"Return a description of the command and how to use it. Example @impl true def description , do : &quot;Cleanup messages.&quot;","ref":"Nosedrum.Command.html#c:description/0","title":"Nosedrum.Command.description/0","type":"callback"},{"doc":"An optional callback that can be used to parse the arguments into something more usable. For example, one might want to use OptionParser along with the arguments to create a more customized command. This command receives the command arguments (without any prefix, command, or if applicable, subcommand), and should return whatever the command/2 function should be passed as the args argument.","ref":"Nosedrum.Command.html#c:parse_args/1","title":"Nosedrum.Command.parse_args/1","type":"callback"},{"doc":"Return a list of predicates that must pass before this command is invoked. This is expected to be invoked by the command processor. Predicates retrieve the message which should be checked for allowance to invoke the command. Predicates are evaluated lazily: For instance, you can have a command which uses a predicate ensuring the command can only be invoked on guilds (for example, by checking whether msg.guild_id == nil ), and predicates afterwards do not need to check whether the command was invoked on a guild. Return value If the predicate allows the invoking user to issue the command, it should return :passthrough . If the user has no permission to execute the command, a pair in the form {:noperm, reason} should be returned, where reason is a concise description of why the user is not allowed to execute the command. {:error, reason} should be returned when the predicate was not able to determine allowance of the invoking user. Example def is_bot ( message ) do if message . author . bot do :passthrough else { :noperm , &quot;sorry, only bots allowed&quot; } end end @impl true def predicates , do : [ &amp; is_bot / 1 ]","ref":"Nosedrum.Command.html#c:predicates/0","title":"Nosedrum.Command.predicates/0","type":"callback"},{"doc":"Return a list of possible ways to use the command (or its subcommands). This is separate from the description/0 callback since the command processor is expected to prepend usage strings with its configured prefix. Example @impl true def usage do [ &quot;clean &lt;amount:int&gt;&quot; , &quot;clean &lt;options...&gt;&quot; ] end","ref":"Nosedrum.Command.html#c:usage/0","title":"Nosedrum.Command.usage/0","type":"callback"},{"doc":"Invoker modules process messages from Discord. They determine the following things: whether the message is a valid command whether the author is permitted to issue the command. When both of these conditions are met, the command callback function is invoked.","ref":"Nosedrum.Invoker.html","title":"Nosedrum.Invoker","type":"behaviour"},{"doc":"Called by consumers when a message arrives. This is the main entry point for invokers: from here on they check whether the message could contain a valid command based on their configured prefix and ask the selected Nosedrum.Storage whether a command exists. If it exists, they proceed to invoke it using only the arguments to the message, with bot prefix and command invocation removed from the message. The second argument, storage , determines which storage implementation the command invoker should use. A command invoker implementation can supply this argument by default if applicable. storage_process is passed along to the given storage and determines which storage process, ETS table, or similar is used.","ref":"Nosedrum.Invoker.html#c:handle_message/3","title":"Nosedrum.Invoker.handle_message/3","type":"callback"},{"doc":"A cache for recent messages sent on guilds. Usage of implementations of this module is useful when your bot needs to access recently sent messages without hogging the API. One use case would be bots with an integrated mod log that want to show a before &amp; after comparison of edited messages.","ref":"Nosedrum.MessageCache.html","title":"Nosedrum.MessageCache","type":"behaviour"},{"doc":"Consume the given message in the cache process identified by cache . Whether this is done asynchronously or synchronously depends on the implementation.","ref":"Nosedrum.MessageCache.html#c:consume/2","title":"Nosedrum.MessageCache.consume/2","type":"callback"},{"doc":"Retrieve a single message for the given guild_id with the specified message_id from the cache process identified by cache . If no message with the given ID is cached, nil is returned.","ref":"Nosedrum.MessageCache.html#c:get/3","title":"Nosedrum.MessageCache.get/3","type":"callback"},{"doc":"Retrieve up to limit messages for the given guild_id from the cache process identified by cache . If limit is :infinity , return all messages for that guild from the cache. If the guild is not cached, an empty message list is returned.","ref":"Nosedrum.MessageCache.html#c:recent_in_guild/3","title":"Nosedrum.MessageCache.recent_in_guild/3","type":"callback"},{"doc":"Update the given message in the cache, if present. Whether this is done asynchronously or synchronously depends on the implementation.","ref":"Nosedrum.MessageCache.html#c:update/2","title":"Nosedrum.MessageCache.update/2","type":"callback"},{"doc":"Storages contain commands and are used by command invokers to look up commands. How you start a storage is up to the module itself - what is expected is that storage modules implement the behaviours documented in this module. The public-facing API of storage modules takes an optional argument, the storage process or other information used to identify the storage such as an ETS table name.","ref":"Nosedrum.Storage.html","title":"Nosedrum.Storage","type":"behaviour"},{"doc":"Add a new command under the given path . If a command has the c:Nosedrum.Command.aliases/0 callback defined, they will also be added under path . If the command already exists, no error should be returned.","ref":"Nosedrum.Storage.html#c:add_command/3","title":"Nosedrum.Storage.add_command/3","type":"callback"},{"doc":"Return a mapping of command names to command_group/0 s. For top-level commands, the value should be a string, otherwise, a mapping of subcommand names to subcommand modules as described on command_group/0 s documentation should be returned.","ref":"Nosedrum.Storage.html#c:all_commands/1","title":"Nosedrum.Storage.all_commands/1","type":"callback"},{"doc":"Look up a command group under the specified name . If the command was not found, nil should be returned.","ref":"Nosedrum.Storage.html#c:lookup_command/2","title":"Nosedrum.Storage.lookup_command/2","type":"callback"},{"doc":"Remove the command under the given path . If a command has the c:Nosedrum.Command.aliases/0 callback defined, they will also be removed under path . If the command does not exist, no error should be returned.","ref":"Nosedrum.Storage.html#c:remove_command/2","title":"Nosedrum.Storage.remove_command/2","type":"callback"},{"doc":"A single command module or mapping of subcommand names to command groups. In addition to subcommand names, the key :default can be specified by the module. :default should be invoked when none of the subcommands in the map match.","ref":"Nosedrum.Storage.html#t:command_group/0","title":"Nosedrum.Storage.command_group/0","type":"type"},{"doc":"The &quot;invocation path&quot; of the command. The public-facing API of storage modules should use this in order to allow users to identify the command they want to operate on. Usage To identify a single command, use a single element list, such as [&quot;echo&quot;] . To identify a subcommand, use a pair, such as [&quot;infraction&quot;, &quot;search&quot;] . To identify the default subcommand invoked when no matching subcommand is found, specify the group name first, then :default , such as [&quot;tags&quot;, :default] .","ref":"Nosedrum.Storage.html#t:command_path/0","title":"Nosedrum.Storage.command_path/0","type":"type"},{"doc":"An OptionParser.split/1 -based command processor. This parser supports a single prefix configured via the nosedrum.prefix configuration variable: config :nosedrum , prefix : &quot;!&quot; The default prefix is . , and the prefix are looked up at compilation time due to the nature of Elixir's binary matching. This means that if you change your prefix, you need to recompile this module, usually using mix deps.compile --force nosedrum . This invoker checks predicates and returns predicate failures to the caller.","ref":"Nosedrum.Invoker.Split.html","title":"Nosedrum.Invoker.Split","type":"module"},{"doc":"Handle the given message. This involves checking whether the message starts with the given prefix, splitting the message into command and arguments, looking up a candidate command, and finally resolving it and invoking the module. Arguments message : The message to handle. storage : The storage implementation the command invoker should use. storage_process : The storage process, ETS table, or similar that is used by the storage process. For instance, this allows you to use different ETS tables for the Nosedrum.Storage.ETS module if you wish. Return value Returns :ignored if one of the following applies: The message does not start with the configured prefix. The message only contains the configured prefix. No command could be looked up that matches the command the message invokes. Examples iex&gt; Nosedrum.Invoker.Split . handle_message ( %{ content : &quot;foo&quot; } ) :ignored iex&gt; Nosedrum.Invoker.Split . handle_message ( %{ content : &quot;.&quot; } ) :ignored","ref":"Nosedrum.Invoker.Split.html#handle_message/3","title":"Nosedrum.Invoker.Split.handle_message/3","type":"function"},{"doc":"An Agent -based message cache. Note that using this cache across your entire bot can easily lead to the cache becoming a bottleneck, since an agent is a single process. Instead, you might want to use this cache across several smaller guilds to avoid overloading a single process with events from multiple guilds. Usage defmodule MyBot.Application do use Application def start ( type , args ) do children = [ %{ id : MessageCache , start : { Nosedrum.MessageCache.Agent , :start_link , [ [ name : Nosedrum.MessageCache.Agent ] ] } } ] end end If using multiple instances of this agent, you can pass the agent process reference as the last argument to the individual functions that this implementation provides. By default, this uses Nosedrum.MessageCache.Agent as the agent reference, requiring you to specify the name: in your application callback. Configuration By default, the cache keeps up to 200 messages in the cache per guild. To configure this, you can pass the limit: option as an argument to the process, such as: defmodule MyBot.Application do use Application def start ( type , args ) do children = [ %{ id : MessageCache , start : { Nosedrum.MessageCache.Agent , :start_link , [ [ name : Nosedrum.MessageCache.Agent , limit : 50 ] ] } } ] end end This will ensure that no more than 50 messages are kept in cache per guild.","ref":"Nosedrum.MessageCache.Agent.html","title":"Nosedrum.MessageCache.Agent","type":"module"},{"doc":"An ETS-table based message cache. This module does not implement any cache expiration: by default, the table fills up infinitely. One possibility of keeping messages under control would be to delete all messages sent before a certain date. Since the key of table entries are message IDs, you can use :ets.select_delete/2 to accomplish this. For example, to delete all messages older than one day, you could use the following construct: now = DateTime . utc_now ( ) one_day_ago = DateTime . add ( now , - 1 , :day ) snowflake = Nostrum.Snowflake . from_datetime! ( one_day_ago ) match_spec = [ { { :&quot;$1&quot; , :_ , :_ , :_ , :_ } , [ { :&lt; , :&quot;$1&quot; , snowflake } ] , [ true ] } ] :ets . select_delete ( :nosedrum_message_cache , match_spec ) For deleting in different ways such as by guild ID, use :ets.fun2ms/1 to generate match specifications from functions. You can use the GenServer.call(cache_process, :tid) to obtain the internal table ID when not using a named table.","ref":"Nosedrum.MessageCache.ETS.html","title":"Nosedrum.MessageCache.ETS","type":"module"},{"doc":"See Nosedrum.MessageCache.consume/2 . Returns the result of :ets.insert .","ref":"Nosedrum.MessageCache.ETS.html#consume/2","title":"Nosedrum.MessageCache.ETS.consume/2","type":"function"},{"doc":"Initialize the ETS message cache. By default, the table used for storing messages is a named table with the name :nosedrum_message_cache . The table reference is stored internally as the state of this process, the public-facing API functions default to using the table name to access the module.","ref":"Nosedrum.MessageCache.ETS.html#start_link/3","title":"Nosedrum.MessageCache.ETS.start_link/3","type":"function"},{"doc":"See Nosedrum.MessageCache.update/2 . Returns the result of :ets.insert/2 .","ref":"Nosedrum.MessageCache.ETS.html#update/2","title":"Nosedrum.MessageCache.ETS.update/2","type":"function"},{"doc":"An implementation of the Nosedrum.Storage behaviour based on ETS tables. This module needs to be configured as part of your supervision tree as it spins up a GenServer which owns the command table. If you want to obtain the table ID of the internal ETS table, send a call with the message :tid .","ref":"Nosedrum.Storage.ETS.html","title":"Nosedrum.Storage.ETS","type":"module"},{"doc":"Initialize the ETS command storage. By default, the table used for storing commands is a named table with the name nosedrum_commands . The table reference is stored internally as the state of this process, the public-facing API functions default to using the table name to access the module.","ref":"Nosedrum.Storage.ETS.html#start_link/3","title":"Nosedrum.Storage.ETS.start_link/3","type":"function"}]